# makefile for kmmalloc V.2.3 or later on Turbo C or Borland C and their make
#	1996/09 kmatsui,	2003/09	kmatsui
#
# Invoke make as follows--
#	make [-D<package> <opt>] [-DMM=<m> || -DWIN32] [-DNO_LIST]
#		<package> should be KMMALLOC, DLMALLOC
#				or BORLANDC (for Borland C only).
#		WIN32 mode needs -DWIN32, MS-DOS mode needs -DMM=<m>.
#		<m> should be one of the s, m, c, l, h.
#		Make of large data memory models needs an option -DLARGE.
# To test and install kmmalloc, invoke the following two or four makes (for
# 		each memory models if and only on MS-DOS).
#	make -DKMMALLOC [-DWIN32 || -DMM=<m> [-DLARGE]] [-DNO_LIST]
#	make -DKMMALLOC -D_MEM_DEBUG [-DWIN32 [-DNO_STDOUT] 	\
#			|| -DMM=<m> [-DLARGE]] [-DNO_LIST]
# This makefile always re-compiles the source files not to mangle different
#	packages.

CFLAGS = -G -d -DPROTO=1
SRCS = memory.c get_env.c listheap.c
OBJS = memory.obj get_env.obj listheap.obj
!if 	$d( WIN32)
# console application
CC = bcc32
CFLAGS = $(CFLAGS) -WC -a4
# output file-name
MEMLIST = memlist
!else
CC = bcc
# CC = tcc
CFLAGS = $(CFLAGS) -a2
!if 	!$d( MM)
# memory model
MM = s
!endif
CFLAGS = $(CFLAGS) -m$(MM)
!endif

# Adjust to your system
!if 	!$d( LIBDIR)
LIBDIR = \BCC55\LIB
# LIBDIR = \TC2\LIB
!endif
# Set your favorite pager
PAGER = notepad
LIST =

!if 	$d( KMMALLOC)
CMACRO = -DKMMALLOC=1
!if 	$d( _MEM_DEBUG)
CMACRO = $(CMACRO) -D_MEM_DEBUG=1
!if 	$d( NO_STDOUT)
CMACRO = $(CMACRO) -DNO_STDOUT=1
!if 	$d( WIN32)
LIB = kmmalloc_debug_nostdout
!else
LIB = km_d_n_
!endif
!else
!if		$d( WIN32)
LIB = kmmalloc_debug
!else
LIB = kmma_d_
!endif
!endif		# $d( NO_STDOUT)
!else
!if 	$d( WIN32)
LIB = kmmalloc
!else
LIB = kmmall_
!endif
!endif		# $d( _MEM_DEBUG)
!if 	!$d( NO_LIST)
LIST = $(CMACRO)
!endif

!elif	$d( BORLANDC)
!if 	!$d( NO_LIST)
LIST = -DBORLANDC=1 listheap.c
!endif
LIB = c

!elif	$d( WIN32) && $d( DLMALLOC)
!if		!$d( NO_LIST)
LIST = -DDLMALLOC=1 listheap.c
!endif

!else
CMACRO =
LIB = c
!endif

!if 	$d( WIN32)
!if 	$d( KMMALLOC)
LIB = $(LIB)32
!elif	$d( DLMALLOC)
LIB = dlmalloc
!else
LIB = import32
!endif
!else
LIB = $(LIB)$(MM)
!endif

!if 	$d( LARGE)
LARGE = -DLARGE=1
!else
LARGE =
!endif

!if 	$d( KMMALLOC)

all:	lib install test time

lib:
	$(CC) $(CFLAGS) $(CMACRO) -c $(SRCS)
	-del $(LIB).lib
	tlib $(LIB).lib +memory +get_env +listheap

install:
	copy /b $(LIB).lib $(LIBDIR)

!else

all:	test time

!endif

clean:
	-del *.obj
	-del *.exe
	-del *.lib
	-del *.out
	-del *.bak
!if 	$d( NO_STDOUT)
	-del $(MEMLIST)
!endif

test:
	$(CC) $(CFLAGS) $(LARGE) -etest_mem $(LIST) test_mem.c $(LIB).lib
	-test_mem
!if 	$d( NO_STDOUT) && $d( KMMALLOC)
	$(PAGER) $(MEMLIST)
!else
	$(PAGER) test_mem.out
!endif

time:
	$(CC) $(CFLAGS) $(LARGE) -etime_mem $(LIST) time_mem.c $(LIB).lib
# Use 'pipe' instead of redirection which TC or BC4's make can't handle.
	-time_mem > time_mem.out
!if 	$d( NO_STDOUT) && $d( KMMALLOC) && !$d( NO_LIST)
	$(PAGER) $(MEMLIST)
!endif
	$(PAGER) time_mem.out
