# makefile for kmmalloc V.2.2 or later on DJGPP and GNU make
#   1996/09 kmatsui,	2003/09 kmatsui
#
# Invoke make as follows--
#   make [PACKAGE=<package>] [LIST=]
#       <package> should be KMMALLOC or BSD_MALLOC.
#       Malloc library of BSD_MALLOC should be installed beforehand.
# To install and test kmmalloc, invoke the following two makes.
#   make PACKAGE=KMMALLOC [LIST=]
#   make PACKAGE=KMMALLOC _MEM_DEBUG=1 [LIST=]
# To test resident malloc() do just
#   make
# This makefile always re-compiles the source files not to mangle different
#   packages.

CC ?= gcc
CFLAGS = -O2 -DPROTO
SRCS = memory.c get_env.c listheap.c
OBJS = memory.o get_env.o listheap.o

# Adjust to your system.
LIBDIR = D:\DJ112\LIB

PACKAGE =
ifneq   ($(PACKAGE), )
    CMACRO = -D$(PACKAGE)
else
    CMACRO =
endif

ifeq    ($(PACKAGE), KMMALLOC)
ifneq   ($(_MEM_DEBUG), )
    CMACRO += -D_MEM_DEBUG=$(_MEM_DEBUG)
    LIB = kmm_d
else
    LIB = kmmal
endif
    LIST = $(CMACRO)
    CLIST = $(LIST)

else
ifeq    ($(PACKAGE), BSD_MALLOC)
# BSD or GNU malloc compiled with -DMSTATS (maybe -DDEBUG -DRCHECK also)
    LIB = mallo

else
    CMACRO =
    LIB = c
    CLIST =
endif
endif

# 'LIST=' option of make for these packages need to revise list() in
# test_mem.c and time_mem.c
ifeq    ($(PACKAGE), BSD_MALLOC)
    LIST = listheap.c
    CLIST += $(CMACRO) $(LIST)
endif

ifeq    ($(PACKAGE), KMMALLOC)

all:    lib install test time

lib:
	$(CC) $(CFLAGS) $(CMACRO) -c $(SRCS)
	ar -rs lib$(LIB).a $(OBJS)

install:
	copy /b lib$(LIB).a $(LIBDIR)

else

all:    test time

endif

clean:
	-del *.o
	-del *.a
	-del *.out
	-del *.bak
	-del test_mem
	-del time_mem

test:
	$(CC) $(CFLAGS) -otest_mem $(CLIST) test_mem.c -l$(LIB)
	-go32 test_mem
	more < test_mem.out

time:
	$(CC) $(CFLAGS) -otime_mem $(CLIST) time_mem.c -l$(LIB)
	go32 time_mem > time_mem.out
	more < time_mem.out
