# makefile for kmmalloc V.2.* on BSD and UCB make
#   1996/09 kmatsui,	2003/09 kmatsui
#
# Invoke make as follows--
#   make [PACKAGE=<package>] [LIST=]
#       <package> should be one of KMMALLOC, DB_MALLOC, DMALLOC, BSD_MALLOC,
#           PHK_MALLOC, DLMALLOC or LEAK.
#       Malloc library of other than KMMALLOC should be installed beforehand.
# To install and test kmmalloc, invoke the following two makes.
#   make PACKAGE=KMMALLOC [LIST=]
#   make PACKAGE=KMMALLOC -D_MEM_DEBUG [LIST=]
# To test resident malloc() do just
#   make
# This makefile always re-compiles the source files not to mangle different
#   packages.

MORE ?= more
CC = gcc
CFLAGS = -O2 -DPROTO
SRCS = memory.c get_env.c listheap.c
OBJS = memory.o get_env.o listheap.o
SOBJS = memory.so get_env.so listheap.so

# Adjust to your system.
LIBDIR = /usr/local/lib
SHLIB_MAJOR ?= 2
SHLIB_MINOR ?= 5

PACKAGE =
.if     !empty(PACKAGE)
    CMACRO = -D$(PACKAGE)
.else
    CMACRO =
.endif

.if     $(PACKAGE) == KMMALLOC
.if     defined( _MEM_DEBUG)
    CMACRO += -D_MEM_DEBUG
    LIB = kmmalloc_debug
.else
    LIB = kmmalloc
.endif
    LIBSO = lib$(LIB).so.$(SHLIB_MAJOR).$(SHLIB_MINOR)
    LIST = $(CMACRO)
    CLIST = $(LIST)

.elif   $(PACKAGE) == DB_MALLOC
    LIB = dbmalloc
    CLIST = -I../dbmalloc
.elif   $(PACKAGE) == DMALLOC
    LIB = dmalloc
    CLIST = -I/usr/local/include
.elif   $(PACKAGE) == BSD_MALLOC
# BSD or GNU malloc compiled with -DMSTATS (maybe -DDEBUG -DRCHECK also)
    LIB = malloc_mstats
.elif   $(PACKAGE) == PHK_MALLOC
# error checking version of phk malloc
    LIB = phkmalloc_extra
    CMACRO += -DMALLOC_EXTRA_SANITY -DMALLOC_STATS
.elif   $(PACKAGE) == DLMALLOC
    LIB = dlmalloc
.elif   $(PACKAGE) == LEAK
    LIB = c
    CLIST = -I../leak
.else
    LIB = c
    CLIST =
.endif

# 'LIST=' option of make for these packages need to revise list() in
# test_mem.c and time_mem.c
.if     $(PACKAGE) == DB_MALLOC || $(PACKAGE) == DMALLOC        \
        || $(PACKAGE) == BSD_MALLOC || $(PACKAGE) == PHK_MALLOC \
        || $(PACKAGE) == DLMALLOC || $(PACKAGE) == LEAK
    LIST = listheap.c
    CLIST += $(CMACRO) $(LIST)
.endif
.if     $(PACKAGE) == LEAK
    CLIST += ../leak/leak.o
.endif

.if     $(PACKAGE) == KMMALLOC

all:    lib install test time

lib:    lib_a lib_so

lib_a:
	$(CC) $(CFLAGS) $(CMACRO) -c $(SRCS)
	ar -q lib$(LIB).a $(OBJS)
	ranlib lib$(LIB).a

# Shared library
.SUFFIXES: .so
$(SOBJS): $(SRCS)
	$(CC) $(CFLAGS) $(CMACRO) -c -fpic -o$*.so $*.c
lib_so: $(SOBJS)
	ld -Bshareable -o$(LIBSO) $(SOBJS)
	chmod a+x $(LIBSO)
	rm *.so

install:
	mv lib$(LIB).a $(LIBSO) $(LIBDIR)
	ln -sf $(LIBDIR)/$(LIBSO) $(LIBDIR)/lib$(LIB).so

.else

all:    test time

.endif

clean:
	-rm *.o *.a *.so *.so.* *.out *.err *.db *_mem *.core

test:
	$(CC) $(CFLAGS) -otest_mem $(CLIST) test_mem.c -L$(LIBDIR) -l$(LIB)
	-./test_mem > test_mem.out 2> test_mem.err
	$(MORE) test_mem.err
	$(MORE) test_mem.out

time:
	$(CC) $(CFLAGS) -otime_mem $(CLIST) time_mem.c -L$(LIBDIR) -l$(LIB)
	-./time_mem > time_mem.out
	$(MORE) time_mem.out
