# makefile for kmmalloc V.2.* on MinGW and GNU make
# 2007/07 kmatsui
#
# Invoke make as follows--
#   make [PACKAGE=<package>] [LIST=]
#       <package> should be one of KMMALLOC, DB_MALLOC, DMALLOC, BSD_MALLOC,
#           PHK_MALLOC, DLMALLOC or LEAK.
#       Malloc library of other than KMMALLOC should be installed beforehand.
# To install and test kmmalloc, invoke the following two makes.
#   make PACKAGE=KMMALLOC [LIST=]
#   make PACKAGE=KMMALLOC _MEM_DEBUG=1 [LIST=]
# To test resident malloc() do just
#   make
# This makefile always re-compiles the source files not to mangle different
#   packages.

MORE ?= less
CC = gcc
CFLAGS = -O2 -DPROTO # -v
SRCS = memory.c get_env.c listheap.c
OBJS = memory.o get_env.o listheap.o
SOBJS = memory.so get_env.so listheap.so

LIBDIR = /usr/local/lib

PACKAGE =
ifneq	($(PACKAGE), )
	CMACRO = -D$(PACKAGE)
else
	CMACRO =
endif

ifeq	($(PACKAGE), KMMALLOC)
ifneq	($(_MEM_DEBUG), )
    CMACRO += -D_MEM_DEBUG=$(_MEM_DEBUG)
    LIB = kmmalloc_debug
else
    LIB = kmmalloc
endif
    LIST = $(CMACRO)
    CLIST = $(LIST)

else
    LIB = c
    CLIST =
endif

# 'LIST=' option of make for these packages need to revise list() in
# test_mem.c and time_mem.c
ifeq    ($(PACKAGE), KMMALLOC)

all:    lib install test time

lib:    lib_a

lib_a:
	$(CC) $(CFLAGS) $(CMACRO) -c $(SRCS)
	ar -rsv lib$(LIB).a $(OBJS)

install:
	mv lib$(LIB).a $(LIBDIR)

else

all:    test time

endif

clean:
	-rm *.o *.exe *.a *.out *.err *_mem *.core

test:
	$(CC) $(CFLAGS) -otest_mem $(CLIST) test_mem.c -L$(LIBDIR) -l$(LIB)
	-./test_mem > test_mem.out 2> test_mem.err
	$(MORE) test_mem.err
	$(MORE) test_mem.out

time:
	$(CC) $(CFLAGS) -otime_mem $(CLIST) time_mem.c -L$(LIBDIR) -l$(LIB)
	-./time_mem > time_mem.out
	$(MORE) time_mem.out
